@@ -265,7 +265,7 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
       str(color_fn);
       str(color_del);
       str(color_off);
-      chr('\n');
+      nl();
       pathname = NULL;
     }
     else
@@ -907,6 +907,9 @@ void Output::format(const char *format, const char *pathname, const std::string&
         break;
 
       case '~':
+#ifdef OS_WIN
+        chr('\r');
+#endif
         chr('\n');
         break;
 
@@ -1255,6 +1258,9 @@ void Output::format_invert(const char *format, const char *pathname, const std::
         break;
 
       case '~':
+#ifdef OS_WIN
+        chr('\r');
+#endif
         chr('\n');
         break;
 
