@@ -500,11 +500,16 @@ class Output {
     chr('0' + (b & 7));
   }
 
-  // output a new line and flush if --line-buffered
-  void nl()
+  // output a newline (platform-specific conditional "\r\n" or "\n"); flush if --line-buffered.
+  void nl(bool lf_only = false)
   {
+    if (!lf_only)
+#ifdef OS_WIN
+      chr('\r');
+#else
+      ;
+#endif
     chr('\n');
-
     check_flush();
   }
 
